@@ -28,9 +28,11 @@ import (
 	v1 "github.com/prometheus/client_golang/api/prometheus/v1"
 )
 
+const DemoPrometheusURL = "https://demo.prometheus.io:443"
+
 func ExampleAPI_query() {
 	client, err := api.NewClient(api.Config{
-		Address: "http://demo.robustperception.io:9090",
+		Address: DemoPrometheusURL,
 	})
 	if err != nil {
 		fmt.Printf("Error creating client: %v\n", err)
@@ -53,7 +55,7 @@ func ExampleAPI_query() {
 
 func ExampleAPI_queryRange() {
 	client, err := api.NewClient(api.Config{
-		Address: "http://demo.robustperception.io:9090",
+		Address: DemoPrometheusURL,
 	})
 	if err != nil {
 		fmt.Printf("Error creating client: %v\n", err)
@@ -104,7 +106,7 @@ func (u userAgentRoundTripper) RoundTrip(r *http.Request) (*http.Response, error
 
 func ExampleAPI_queryRangeWithUserAgent() {
 	client, err := api.NewClient(api.Config{
-		Address:      "http://demo.robustperception.io:9090",
+		Address:      DemoPrometheusURL,
 		RoundTripper: userAgentRoundTripper{name: "Client-Golang", rt: api.DefaultRoundTripper},
 	})
 	if err != nil {
@@ -133,9 +135,13 @@ func ExampleAPI_queryRangeWithUserAgent() {
 
 func ExampleAPI_queryRangeWithBasicAuth() {
 	client, err := api.NewClient(api.Config{
-		Address: "http://demo.robustperception.io:9090",
+		Address: DemoPrometheusURL,
 		// We can use amazing github.com/prometheus/common/config helper!
-		RoundTripper: config.NewBasicAuthRoundTripper("me", "definitely_me", "", "", api.DefaultRoundTripper),
+		RoundTripper: config.NewBasicAuthRoundTripper(
+			config.NewInlineSecret("me"),
+			config.NewInlineSecret("definitely_me"),
+			api.DefaultRoundTripper,
+		),
 	})
 	if err != nil {
 		fmt.Printf("Error creating client: %v\n", err)
@@ -163,9 +169,52 @@ func ExampleAPI_queryRangeWithBasicAuth() {
 
 func ExampleAPI_queryRangeWithAuthBearerToken() {
 	client, err := api.NewClient(api.Config{
-		Address: "http://demo.robustperception.io:9090",
+		Address: DemoPrometheusURL,
 		// We can use amazing github.com/prometheus/common/config helper!
-		RoundTripper: config.NewAuthorizationCredentialsRoundTripper("Bearer", "secret_token", api.DefaultRoundTripper),
+		RoundTripper: config.NewAuthorizationCredentialsRoundTripper(
+			"Bearer",
+			config.NewInlineSecret("secret_token"),
+			api.DefaultRoundTripper,
+		),
+	})
+	if err != nil {
+		fmt.Printf("Error creating client: %v\n", err)
+		os.Exit(1)
+	}
+
+	v1api := v1.NewAPI(client)
+	ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
+	defer cancel()
+	r := v1.Range{
+		Start: time.Now().Add(-time.Hour),
+		End:   time.Now(),
+		Step:  time.Minute,
+	}
+	result, warnings, err := v1api.QueryRange(ctx, "rate(prometheus_tsdb_head_samples_appended_total[5m])", r)
+	if err != nil {
+		fmt.Printf("Error querying Prometheus: %v\n", err)
+		os.Exit(1)
+	}
+	if len(warnings) > 0 {
+		fmt.Printf("Warnings: %v\n", warnings)
+	}
+	fmt.Printf("Result:\n%v\n", result)
+}
+
+func ExampleAPI_queryRangeWithAuthBearerTokenHeadersRoundTripper() {
+	client, err := api.NewClient(api.Config{
+		Address: DemoPrometheusURL,
+		// We can use amazing github.com/prometheus/common/config helper!
+		RoundTripper: config.NewHeadersRoundTripper(
+			&config.Headers{
+				Headers: map[string]config.Header{
+					"Authorization": {
+						Values: []string{"Bearer secret"},
+					},
+				},
+			},
+			api.DefaultRoundTripper,
+		),
 	})
 	if err != nil {
 		fmt.Printf("Error creating client: %v\n", err)
@@ -193,7 +242,7 @@ func ExampleAPI_queryRangeWithAuthBearerToken() {
 
 func ExampleAPI_series() {
 	client, err := api.NewClient(api.Config{
-		Address: "http://demo.robustperception.io:9090",
+		Address: DemoPrometheusURL,
 	})
 	if err != nil {
 		fmt.Printf("Error creating client: %v\n", err)
